#include "theme.h"
#include "input.h"
#include "menudata.h"
#include "menuhandler.h"
#include "history.h"
#include "config.h"


SDL_Surface *initScreen(int opts, vector size) {
	SDL_Surface *screen;
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_JOYSTICK)<0) {
		fprintf(stderr,"Couldn't init SDL!");
		exit(1);
	}
	atexit(SDL_Quit);

	SDL_WM_SetCaption("XboxMenu","XboxMenu");
	screen=SDL_SetVideoMode(size.x,size.y,32,SDL_DOUBLEBUF|SDL_SWSURFACE|opts);
	if (screen == NULL) {
		fprintf(stderr,"Couldn't create surface with the size of the theme! %s\n",SDL_GetError());
		exit(1);
	}
//	SDL_WarpMouse(size.x,size.y);
	return screen;
}


int main(int argc, char** argv) {
	char menuFile[1024];
	char themeDir[1024];
	char header[512];
	char args[1024];
	int hasArg;
	int selected=0;
	int escaped;
	int dir=1;
	int x;
	int opts=0;
	int scalex, scaley;
	SDL_Surface *screen;
	historyHandle hist=historyLoad(HIST_FILE);

	strcpy(menuFile, DEFAULT_MENU);
	strcpy(themeDir, DEFAULT_THEME);
	scalex=0; scaley=0;
	for (x=1; x<argc; x++) {
		if (strcmp(argv[x],"-m")==0 && argc>x) {
			x++;
			strcpy(menuFile+1,argv[x]);
		} else if (strcmp(argv[x],"-t")==0 && argc>x) {
			x++;
			strcpy(themeDir,argv[x]);
		} else if (strcmp(argv[x],"-f")==0) {
			opts|=SDL_FULLSCREEN;
		} else if (strcmp(argv[x],"-x")==0 && argc>x) {
			x++;
			scalex=atoi(argv[x]);
		} else if (strcmp(argv[x],"-y")==0 && argc>x) {
			x++;
			scaley=atoi(argv[x]);
		} else {
			printf("Usage: %s [-t themedir] [-m menufile.mnu] [-x width -y height] [-f]\n",argv[0]);
			printf(" -f: run fullscreen\n");
			exit(0);
		}
	}

	hasArg=0;

	themeInitEngine();
	themeHandle theme=themeLoad(themeDir);
	vector size;
	themeGetSize(theme, &size);
	if (scalex!=0) size.x=scalex;
	if (scaley!=0) size.y=scaley;
	screen=initScreen(opts, size);
	themeOptimizeGfx(theme);
	inputInit();

	if (historyLen(hist)==0) {
		dir=1;
	} else {
		historyPop(hist, menuFile, &hasArg, args, &selected);
		dir=-1;
	}

	while(1) {
		escaped=0;
		menuDataHandle menudata=menuDataRetrieve(menuFile, header, strlen(args)!=0?args:NULL);
		if (menudata==NULL) {
			//MenuData couldn't handle it -> means we need to run it. Print it to stdout and exit.
			historyWrite(hist, HIST_FILE);
			printf("%s\n",menuFile);
			exit(0);
		}
	
		//Show menu so the user can select an item :)
		selected=menuHandlerDoMenu(menudata, theme, screen, header, selected, dir);
		if (selected>=0) { //Item chosen.
			//Push on stack.
			historyPush(hist, menuFile, hasArg, args, selected);
			strcpy(menuFile, menuDataGetActionFor(menudata, selected));
			fprintf(stderr,"Selected: %i: %s\n",selected,menuFile);
			selected=0;
			dir=1;
			strcpy(args,"");
			if (menuFile[0]=='?') { 	//Waah! We need more info to make the next choice!
				//Is a query. We have to ask the user for the param first, before passing it on
				//to the menuDataHandler. Parse header out and restore menyFile.
				char *qStart=strstr(menuFile,"|");
				*qStart=0;
				strcpy(header, menuFile+1);
				menuFile[0]='|';
				strcpy(menuFile+1, qStart+1);
				escaped=!menuHandlerGetInput(theme, screen, header, args, 1024);
				hasArg=1;
			} else {
				hasArg=0;
			}
		}
		if (selected<0 || escaped) {//Go back.
			escaped=0;
			historyPop(hist, menuFile, &hasArg, args, &selected);
			dir=-1;
		}
		menuDataFree(menudata);
	}
}
