#define _GNU_SOURCE
#include "menudata.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/select.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <strings.h>
#include <signal.h>

void applyFilter(menuDataHandle dat) {
	int x,n;
	n=0;
	for (x=0; x<dat->noItems; x++) {
		if (strcasestr(dat->itemName[x], dat->filter)) {
			dat->filteredNames[n]=dat->itemName[x];
			dat->filteredActions[n]=dat->itemAction[x];
			n++;
		}
	}
	dat->noFilteredItems=n;
}

static int tryGetString(menuDataHandle m, char *line, int len) {
	int x, pos;
	if (m->fd<0) return -1;

	//Check for an enter in the buffer.
	pos=0;
	while (m->inBuff[pos]!='\n' && pos<m->inBuffPos) pos++;
	if (pos<m->inBuffPos) {
		//W00t, got a line :) C/p to 'line'.	
		for (x=0; x<pos; x++) {
			if (x<len) line[x]=m->inBuff[x];
		}
		line[x]=0; //zero-terminate
		//Hack: Just an enter -> close fd and exit.
		if (x==0) return -1;
		//Now shift the buffer for the next var.
		for (x=pos+1; x<sizeof(m->inBuff); x++) {
			m->inBuff[x-(pos+1)]=m->inBuff[x];
		}
		m->inBuffPos-=(pos+1);
		applyFilter(m);
		return 1;
	}

	//No enter -> fetch moar data!
	fd_set rfds;
	struct timeval tv;
	int ret;
	FD_ZERO(&rfds); FD_SET(m->fd, &rfds);
	tv.tv_sec=0; tv.tv_usec=0;
	ret=select(m->fd+1, &rfds, NULL, NULL, &tv);
	if (!ret) return 0;

	int r=read(m->fd, m->inBuff+m->inBuffPos, sizeof(m->inBuff)-m->inBuffPos);
	if (r>0) {
		m->inBuffPos+=r;
		//Ok, got more data. Need to parse it in the next call tho'.
		return 0;
	} else {
		//Ok, fd is done for.
		return -1;
	}
}

int makeProcess(char *what, int *out) {
	int pipeIn[2];
	int pipeOut[2];
	int pid;
	
	signal(SIGPIPE, SIG_IGN);
	signal(SIGCHLD, SIG_IGN);
	
	pipe(pipeIn);
	pipe(pipeOut);
	pid=fork();
	if (pid==0) {
		//Child. Exec script.
		close(1);
		dup(pipeIn[1]);
		close(0);
		dup(pipeOut[0]);
		close(pipeIn[0]);
		close(pipeOut[1]);
		execl("/bin/sh", "/bin/sh", "-c", what, NULL);
	} else {
		close(pipeIn[1]);
		close(pipeOut[0]);
		*out=pipeOut[1];
		return pipeIn[0];
	}
	return -1;
}

menuDataHandle menuDataRetrieve(char *whence, char* header, char *args) {
	int argfd;
	menuDataHandle m=malloc(sizeof(menuDataStruct));
	m->fd=-1;
	m->inBuffPos=0;
	m->noItems=0;
	if (whence[0]=='>') {
		m->fd=open(whence+1, O_RDONLY);
	} else if (whence[0]=='|') {
		m->fd=makeProcess(whence+1,&argfd);
		if (args!=NULL) {
			write(argfd, args, strlen(args));
			write(argfd, "\n", 1);
		}
		close(argfd);
	} else {
		return NULL;
	}
	if (m->fd<0) return NULL;
	//Yay, we've got something. Grab the first line.
	while(!tryGetString(m, header, 512)) ;
//	fprintf(stderr,"Got header %s\n",header);
	//1024 items oughtto be enough for a first guess.
	m->itemName=malloc(sizeof(char*)*1024);
	m->itemAction=malloc(sizeof(char*)*1024);
	m->filteredActions=malloc(sizeof(char*)*1024);
	m->filteredNames=malloc(sizeof(char*)*1024);
	strcpy(m->filter,"");
	return m;
}

//If possible, tries to fetch more data from the file/pipe. Returns 0 if waiting
//for data, 1 if all done.
int menuDataGetMore(menuDataHandle m) {
	int r;
	char line[1024];
	if (m->fd==-1) return 1;
	r=tryGetString(m, line, 1024);
	if (r==-1) {
		close(m->fd);
		m->fd=-1;
		return 1;
	}
	if (r) {
		//Got a line! Split line into a name and an action.
		char *p=strstr(line, "#");
		if (p!=NULL) {
			m->itemName[m->noItems]=malloc(p-line+1);
			m->itemAction[m->noItems]=malloc(strlen(p));
			*p=0;
			strcpy(m->itemName[m->noItems], line);
			strcpy(m->itemAction[m->noItems], p+1);
			m->noItems++;
			applyFilter(m);
		}
	}
	return 0;
}

int menuDataGetItemNames(menuDataHandle dat, char*** itemNames) {
	if (strlen(dat->filter)==0) {
		*itemNames=dat->itemName;
		return dat->noItems;
	} else {
		*itemNames=dat->filteredNames;
		return dat->noFilteredItems;
	}
}

void menuDataFree(menuDataHandle h) {
	int n;
	for (n=0; n<h->noItems; n++) {
		free(h->itemName[n]);
		free(h->itemAction[n]);
	}
	free(h->itemName);
	free(h->itemAction);
	free(h->filteredActions);
	free(h->filteredNames);
	free(h);
}

void menuDataSetFilter(menuDataHandle dat, char *filter) {
	strcpy(dat->filter, filter);
	applyFilter(dat);
}

char *menuDataGetActionFor(menuDataHandle dat, int selected) {
	if (strlen(dat->filter)==0) {
		return dat->itemAction[selected];
	} else {
		return dat->filteredActions[selected];
	}
}
