#include "input.h"
#include "menuhandler.h"
#include <SDL/SDL.h>
#include <SDL/SDL_rotozoom.h>

#define MENU_SKIP_BETWEEN_ADDS 0
#define MENU_DELAY_BEFORE_SHOWING_WAITIND	10

SDL_Surface *getScaledDest(themeHandle theme, SDL_Surface *phys) {
	SDL_Surface *screen;
	vector size;
	themeGetSize(theme, &size);
	if (phys->w!=size.x || phys->h!=size.y) {
		//Create a virtual blit dest the size of the theme.
		screen=SDL_CreateRGBSurface(SDL_SWSURFACE, size.x, size.y, 32, 0,0,0,0);
		fprintf(stderr, "We need to scale: %ix%i -> %ix%i.\n", size.x, size.y, phys->w, phys->h);
	} else {
		//Just pass along the physical screen surface.
		screen=phys;
	}
	return screen;
}

void displayScaled(SDL_Surface *phys, SDL_Surface *virt) {
	if (phys!=virt) {
		//The noes, we need to scale.
		SDL_Surface *tmp=zoomSurface(virt, (double)phys->w/(double)virt->w, (double)phys->h/(double)virt->h, 0);
		SDL_BlitSurface(tmp, NULL, phys, NULL);
		SDL_FreeSurface(tmp);
	}
	SDL_Flip(phys);
}


int menuHandlerGetInput(themeHandle theme, SDL_Surface *physScreen, char *header, char *buff, int len) {
	long nextTime;
	int n=0;
	SDL_Surface *screen=getScaledDest(theme, physScreen);
	themeSetInputboxTitle(theme, header);
	themeSetInputbox(theme, buff, 0);
	while(n!=INP_SELECT) {
		nextTime=SDL_GetTicks()+20; //50FPS

		themeDrawInputbox(screen, theme);
		displayScaled(physScreen, screen);
		n=inputGetDirection();
		if (n!=INP_NONE) {
			if (n&INP_CHAR && strlen(buff)<len) {
				buff[strlen(buff)+1]=0;
				buff[strlen(buff)]=n&0x7f;
				themeSetInputbox(theme, buff, 0);
			}
			if (n==INP_BSP && strlen(buff)>0) {
				buff[strlen(buff)-1]=0;
				themeSetInputbox(theme, buff, 0);
			}
			if (n==INP_BACK) {
				return 0;
			}
		}
		nextTime-=SDL_GetTicks();
		if (nextTime>0) SDL_Delay(nextTime);
	}
	return 1;
}



int menuHandlerDoMenu(menuDataHandle menuData, themeHandle theme, SDL_Surface *physScreen, char *header, int preselect, int dir) {
	char searchTerm[1024];
	int noItems=0;
	char** itemNames;
	long nextTime;
	int selected=0;
	int count=0;
	themeNewMenu(theme, header);
	int moveAwayDir=0;
	float menuOffset=0;
	int waitCtr=0;
	SDL_Surface *screen=getScaledDest(theme, physScreen);
	strcpy(searchTerm,"");
	themeSetSearch(theme, searchTerm);
	selected=preselect;

	while(menuOffset<1 && menuOffset>-1) {
		count++;
		nextTime=SDL_GetTicks()+20; //50FPS

		if (count>MENU_SKIP_BETWEEN_ADDS) {
			if (menuDataGetMore(menuData)) waitCtr=0; else waitCtr++;
			if (waitCtr>MENU_DELAY_BEFORE_SHOWING_WAITIND) themeSetWait(theme, 1); else themeSetWait(theme, 0);
			noItems=menuDataGetItemNames(menuData, &itemNames);
			count=0;
		}

		if (moveAwayDir!=0) {
			menuOffset+=menuOffset;
		}

		themeDraw(screen, theme, itemNames, noItems, selected, menuOffset, dir);
		displayScaled(physScreen, screen);

		nextTime-=SDL_GetTicks();
		if (nextTime>0) SDL_Delay(nextTime);
		int n=inputGetDirection();
		if (n!=INP_NONE && moveAwayDir==0) {
			if (n==INP_UP) selected--;
			if (n==INP_DOWN) selected++;
			if (n==INP_PGUP) selected-=themeGetPageSize(theme);
			if (n==INP_PGDWN) selected+=themeGetPageSize(theme);
			if (n==INP_HOME) selected=0;
			if (n==INP_END) selected=noItems-1;
			if (n==INP_BACK) {
				moveAwayDir=1;
				menuOffset=.001;
			}
			if (n==INP_SELECT) {
				moveAwayDir=-1;
				menuOffset=-.001;
			}
			if (n&INP_CHAR && strlen(searchTerm)<1022) {
				searchTerm[strlen(searchTerm)+1]=0;
				searchTerm[strlen(searchTerm)]=n&0x7f;
				themeSetSearch(theme, searchTerm);
				menuDataSetFilter(menuData, searchTerm);
				selected=0;
				dir=1;
			}
			if (n==INP_BSP && strlen(searchTerm)>0) {
				searchTerm[strlen(searchTerm)-1]=0;
				themeSetSearch(theme, searchTerm);
				menuDataSetFilter(menuData, searchTerm);
				selected=0;
			}
			if (selected>=noItems) selected=noItems-1;
			if (selected<0) selected=0;

		nextTime-=SDL_GetTicks();
		if (nextTime>0) SDL_Delay(nextTime);

		}
	}
	if (moveAwayDir==1) return -1;
	return selected;
}

