#!/bin/bash

function getIniVal() {
	cat "$INI" | fgrep -i -A 1024 "[$1]" | fgrep -i "$2=" | cut -d '=' -f 2 | head -n 1
}

function calc() {
	echo "scale=6; $1" | bc -q | cut -d '.' -f 1
}

function scaleX() {
	#scaleX iniSec iniKey
	orgVal=`getIniVal $1 $2`
	scaledVal=`calc "($newX/$sizeX)*$orgVal"`
	echo "$2=$scaledVal"
}

function scaleY() {
	#scaleX iniSec iniKey
	orgVal=`getIniVal $1 $2`
	scaledVal=`calc "($newY/$sizeY)*$orgVal"`
	echo "$2=$scaledVal"
}

function scaleImage {
	size="`file $path/$1 | cut -d ',' -f 2`"
	picX=`echo $size | cut -d 'x' -f 1`
	picY=`echo $size | cut -d 'x' -f 2`
	npicX=`calc "($newX/$sizeX)*$picX"`
	npicY=`calc "($newY/$sizeY)*$picY"`
	echo "Scaling $1 to $2: $picX x $picY -> $npicX x $npicY"
	convert "$path/$1" -resize ${npicX}x${npicY} "$newpath/$1"
}

function scaleImages {
	IFS=','
#	echo "Scaling image seq $1"
	for x in $1; do 
		scaleImage "$x"
	done
}

function passthrough {
	echo "$2=`getIniVal $1 $2`"
}

if [ -z "$4" ]; then
	echo "Usage: $0 srcThemeDir newWidth newHeight destThemeDir"
	exit 0
fi

path=$1
newpath=$4
INI="$path/theme.ini"
bgnd="$path/`getIniVal bgnd image`"
size="`file $bgnd | cut -d ',' -f 2`"
sizeX=`echo $size | cut -d 'x' -f 1`
sizeY=`echo $size | cut -d 'x' -f 2`
newX=$2
newY=$3


mkdir "$newpath"
scaleImage "`getIniVal bgnd image`"
scaleImage "`getIniVal fgnd image`"
scaleImage "`getIniVal ball image`"
scaleImages "`getIniVal input sequence`"
scaleImages "`getIniVal wait sequence`"
scaleImages "`getIniVal search sequence`"

cp "$path/`getIniVal menutext font`" "$newpath/`getIniVal menutext font`"
cp "$path/`getIniVal headertext font`" "$newpath/`getIniVal headertext font`"
cp "$path/`getIniVal search font`" "$newpath/`getIniVal search font`"
cp "$path/`getIniVal input font`" "$newpath/`getIniVal input font`"

{
echo "[bgnd]"
passthrough bgnd image
echo
echo "[fgnd]"
passthrough fgnd image
echo
echo "[ball]"
echo "image=`getIniVal ball image`"
scaleX ball startx
scaleY ball starty
scaleX ball endx
scaleY ball endy
echo
echo "[menutext]"
passthrough menutext font
scaleY menutext size
scaleX  menutext startx
scaleX  menutext margin
scaleY menutext starty
scaleX  menutext endx
scaleY menutext endy
scaleY menutext selectedy
scaleY menutext stride
passthrough menutext color
echo
echo '[headertext]'
passthrough headertext font
scaleY headertext size
scaleX  headertext startx
scaleY headertext starty
scaleX  headertext endx
scaleY headertext endy
passthrough headertext color
echo
echo '[wait]'
scaleX  wait posx
scaleY wait posy
passthrough wait sequence
passthrough wait delay
passthrough wait loopstart
passthrough wait loopend
echo
echo '[search]'
scaleX  search posx
scaleY search posy
passthrough search sequence
passthrough search delay
passthrough search loopstart
passthrough search loopend
scaleX  search textstartx
scaleY search textstarty
scaleX  search textendx
scaleY search textendy
passthrough search font
scaleY search size
passthrough search color
echo
echo '[input]'
scaleX  input posx
scaleY input posy
passthrough input sequence
passthrough input delay
passthrough input loopstart
passthrough input loopend
scaleX  input textstartx
scaleY input textstarty
scaleX  input textendx
scaleY input textendy
passthrough input font
scaleY input size
passthrough input color
echo
echo '[inputtitle]'
scaleX  inputtitle textstartx
scaleY inputtitle textstarty
passthrough inputtitle font
scaleY inputtitle size
passthrough inputtitle color
} > $newpath/theme.ini